TEXTURE *hole_text;

// buffer containing the (u,v) pairs at each pixel
unsigned char *texcoord;

/*
 * position of the centre of the hole along the X axis
 */
float get_x_pos( float f )
{
      return - 16 * sin(  f * M_PI / 256 );
};

/*
 * position of the centre of the hole along the Y axis
 */
float get_y_pos( float f )
{
      return - 16 * sin(  f * M_PI / 256 );
};

/*
 * size of the hole
 */
float get_radius( float f )
{
      return 128;
};

void Hole_Init()
{
 // alloc memory to store 320*200 times u, v
    texcoord = new (unsigned char)[128000];
    long offs = 0;
 // precalc the (u,v) coordinates
    for (int j=-100; j<100; j++) {
        for (int i=-160; i<160; i++)
        {
          // get coordinates of ray that projects through this pixel
             float dx = (float)i / 200;
             float dy = (float)-j / 200;
             float dz = 1;
          // normalize them
             float d = 20/sqrt( dx*dx + dy*dy + 1 );
             dx *= d;
             dy *= d;
             dz *= d;
          // start interpolation at origin
             float x = 0;
             float y = 0;
             float z = 0;
          // set original precision
             d = 16;
          // interpolate along ray
             while (d>0)
             {
                // continue until we hit a wall
                   while (((x-get_x_pos(z))*(x-get_x_pos(z))+(y-get_y_pos(z))*(y-get_y_pos(z)) < get_radius(z)) && (z<1024))
                   {
                       x += dx;
                       y += dy;
                       z += dz;
                   };
                // reduce precision and reverse direction
                   x -= dx;  dx /= 2;
                   y -= dy;  dy /= 2;
                   z -= dz;  dz /= 2;
                   d -= 1;
             }
          // calculate the texture coordinates
             x -= get_x_pos(z);
             y -= get_y_pos(z);
             float ang = atan2( y, x ) * 256 / M_PI;
             unsigned char u = (unsigned char)ang;
             unsigned char v = (unsigned char)z;
          // store texture coordinates
             texcoord[offs] = u;
             texcoord[offs+1] = v;
             offs += 2;
        }
    }
 // load the texture
    hole_text = new TEXTURE( "hole.png" );
};

/*
 * non asm main loop
 */
void Draw_Hole( unsigned char *buf, unsigned char du, unsigned char dv )
{
   long doffs = 0, soffs = 0;
   for (int j=0; j<200; j++) {
       for (int i=0; i<320; i++) {
      // load (u,v) and add displacement
         unsigned char u = texcoord[soffs] + du;
         unsigned char v = texcoord[soffs+1] + dv;
         buf[doffs] = hole_text->location[(v<<8)+u];
         doffs++;
         soffs+=2;
       }
   }
}

void Hole_Run()
{
    bQuit = false;
// run the main loop
    while (!kbhit() && !bQuit)
    {
        // set the colours
          for (int i=0; i<256; i++)
          {
              vga->SetColour( i,
              (unsigned char)((hole_text->palette[i*3]>>2)*(1-flash_coef)+63*flash_coef),
              (unsigned char)((hole_text->palette[i*3+1]>>2)*(1-flash_coef)+63*flash_coef),
              (unsigned char)((hole_text->palette[i*3+2]>>2)*(1-flash_coef)+63*flash_coef) );
          }
          Draw_Hole( vga->page_draw, timer->getCount()>>16, timer->getCount()>>14 );
          vga->Update();
          frameCount++;
    }
}

void Hole_Done()
{
// free memory
    delete texcoord;
}


